#! /bin/sh -

# ----------------------------------------------------------------------------
# Specify if this is a patch package or hotfix package
# If this is patch package, system upgrade record is 
#	$MAJOR_VER.$MINOR_VER-$BUILD_NUM-0-$BUILD_DESCRIPTION
# If this is hotfix package, system upgrade record is 
#       $MIN_REQ_VER-$MIN_REQ_BUILD-$HOTFIX_VER-$BUILD_DESCRIPTION 
# ----------------------------------------------------------------------------
PATCH_PACKAGE=

BUILD_DESCRIPTION=SP1

HOTFIX_VER=2

# -----------------------------------------------------------------------------
# Define exit codes for ServPatch.sh
# -----------------------------------------------------------------------------

# install.sh is executed successfully.
ECODE_INS_OK=0

# install.sh failed.
ECODE_INS_ERR=1

# reboot is requested by install.sh
ECODE_INS_REBOOT=2

# TMSSS does not meet minimum required version build specified in PatchVersion.ini
ECODE_INS_MIN_REQ_FAILED=3

# ----------------------------------------------------------------------------
# Don't change VERSION and BUILD information below, because this information 
# will be replaced by makefile at the compile time.
# ---------------------------------------------------------------------------
MAJOR_VER=1
MINOR_VER=1
BUILD_NUM=1168

# -----------------------------------------------------------------------------
# function: debug_log
#
# @stdin message
# @stdout message
# -----------------------------------------------------------------------------

LOG_INSTALL_SCRIPT=/var/tmcss/patch/pkg/install.log
debug_log () {
    tee -a $LOG_INSTALL_SCRIPT
}

# -----------------------------------------------------------------------------
# Generate temp log
# -----------------------------------------------------------------------------

echo `date -R` | debug_log
echo | debug_log

# -----------------------------------------------------------------------------
# function: retrive integer from string. toward zero is trucated.
#
# $1: string to be converted
# @stdout integer value
# -----------------------------------------------------------------------------

VersionBuildInfo() {
	TMP_MAJOR_VER=`cat $1 | grep '^TMCSSVersion=' | cut -d '=' -f 2 | cut -d '.' -f 1`
	TMP_MINOR_VER=`cat $1 | grep '^TMCSSVersion=' | cut -d '=' -f 2 | cut -d '.' -f 2`
	TMP_BUILD_NUM=`cat $1 | grep '^TMCSSBuild=' | cut -d '=' -f 2`
	echo $[ ($TMP_MAJOR_VER * 100 + $TMP_MINOR_VER ) * 10000 + $TMP_BUILD_NUM ] 
}

MinReqVersionBuild() {
	TMP_MAJOR_VER=`cat $1 | grep '^MinReqVersion=' | cut -d '=' -f 2 | cut -d '.' -f 1`
	TMP_MINOR_VER=`cat $1 | grep '^MinReqVersion=' | cut -d '=' -f 2 | cut -d '.' -f 2`
	TMP_BUILD_NUM=`cat $1 | grep '^MinReqBuild=' | cut -d '=' -f 2`
	echo $[ ($TMP_MAJOR_VER * 100 + $TMP_MINOR_VER ) * 10000 + $TMP_BUILD_NUM ] 
}

# -----------------------------------------------------------------------------
#  Constant Definitions
# -----------------------------------------------------------------------------

TMSSS_VERSION_INI=/var/tmcss/conf/Version.ini
PATCH_VERSION_INI=/var/tmcss/patch/pkg/PatchVersion.ini
PATCH_TEMP_DIR=patchtmp

# ------------------------------------------------------------------------------
# Check for minimum required verison and build
# For all 1.0 Patches, the minimum required version and build are 1.0 1245 build
# For hotfixes, the minumum required version and build are previous one patch
# ------------------------------------------------------------------------------

TMSSS_VERSION_INFO=`VersionBuildInfo $TMSSS_VERSION_INI`
echo "TMSSS version and build = "$TMSSS_VERSION_INFO | debug_log
PATCH_MIN_REQ_VERSION=`MinReqVersionBuild $PATCH_VERSION_INI`
echo "Patch minimum required version and build = "$PATCH_MIN_REQ_VERSION | debug_log

if [ "$TMSSS_VERSION_INFO" -lt "$PATCH_MIN_REQ_VERSION" ]
then
	echo "[ERROR] TMCSS does not meet minimum required version build information" | debug_log
	exit $ECODE_INS_MIN_REQ_FAILED
fi

# ---------------------------------------------------------------------------
# Start to apply patches
# Specify all patches in ALL_PATCHES and make sure the order is correct 
# ---------------------------------------------------------------------------
DIR_CURRENT=`pwd`

RET_EXIT_INSTALL=$ECODE_INS_ERR

apply_patch() {
	echo "Create temp patch directory: $PATCH_TEMP_DIR" | debug_log
	rm -rf $PATCH_TEMP_DIR
	mkdir $PATCH_TEMP_DIR

	echo "tar zxvf $1 into $PATCH_TEMP_DIR" | debug_log
	tar zxvf $1 -C $PATCH_TEMP_DIR/
	
	# change permission recursively
	chmod 777 -vR $PATCH_TEMP_DIR | debug_log

	echo "Change current working directory to $PATCH_TEMP_DIR" | debug_log
	cd $PATCH_TEMP_DIR

	echo "Execute Patch install.sh script" | debug_log
	./install.sh
	RET_EXIT_INSTALL=$?
	echo "Patch return code is $RET_EXIT_INSTALL" | debug_log

	echo "Cat $PATCH_TEMP_DIR/install.log to $LOG_INSTALL_SCRIPT" | debug_log
	cat install.log | debug_log

	echo "Change current working directory back to $DIR_CURRENT" | debug_log 
	cd $DIR_CURRENT

	echo "Remove temp patch directory: $PATCH_TEMP_DIR" | debug_log
	rm -rf $PATCH_TEMP_DIR
}


# Currently, we only have TMSSS 1.0 patch 1
#
# In the future, supposed we are deliverying patch 3 (current patch build), the sample code is like below
#
# if ( system build < patch 1 build ) {
# 	apply_patch patch 1
# 	apply_patch patch 2
#	apply_patch patch 3
# } else if (system build < patch 2 build ) {
#	apply_patch patch 2 
#	apply_patch patch 3
# } else if (system build < current patch build ) {
#	apply_patch patch 3
# }
#

SYSTEM_UPGRADE_HISTORY_FILE=/var/tmcss/conf/SysStatus.ini

CURRENT_PATCH_VERSION_INFO=`VersionBuildInfo $PATCH_VERSION_INI`
echo "Current patch version and build = "$CURRENT_PATCH_VERSION_INFO | debug_log

if [ "$TMSSS_VERSION_INFO" -lt "$CURRENT_PATCH_VERSION_INFO" ]
then
	PATCHES_NEED_APPLY="HotFix-2-$MAJOR_VER.$MINOR_VER-$BUILD_NUM.x86_64.tar.gz"
	
	# Apply TMSSS v1.1 hotfix 1, if hotfix 1 was not applied.
	grep '1.1-1155-1-SP1' $SYSTEM_UPGRADE_HISTORY_FILE
	INSTALLED_HOTFIX_1=$?	
	if [ "$INSTALLED_HOTFIX_1" -ne 0 ]
	then
		PATCHES_NEED_APPLY="HotFix-1-1.1-1164.x86_64.tar.gz "$PATCHES_NEED_APPLY
	fi

	for p in $PATCHES_NEED_APPLY
	do
		echo "Apply patch $p" | debug_log
		apply_patch $p
		if [ "$RET_EXIT_INSTALL" -ne 0 ]
		then
			echo "[ERROR] Apply patch failed" | debug_log
			break
		fi
	done
fi

# -----------------------------------------------------------------------------
# All thing are done, and see if there is a need to inform user
# for rebooting Linux.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# function: read ini value by key
#
# $1 = file name
# $2 = key
# @stdout ini value
# -----------------------------------------------------------------------------

read_ini_value() {
    cat $1 | grep '^'$2'=' | cut -d '=' -f 2
}

update_upgrade_record(){
	echo "Update $SYSTEM_UPGRADE_HISTORY_FILE" | debug_log
	if [ "X$PATCH_PACKAGE" = "X1" ]
	then
		echo "$MAJOR_VER.$MINOR_VER-$BUILD_NUM-0-$BUILD_DESCRIPTION" >> $SYSTEM_UPGRADE_HISTORY_FILE
	else
		MIN_REQ_VER=`read_ini_value $PATCH_VERSION_INI 'MinReqVersion'`
		MIN_REQ_BUILD=`read_ini_value $PATCH_VERSION_INI 'MinReqBuild'`
		PREVIOUS_VER=1
		while [ $PREVIOUS_VER -le $HOTFIX_VER ]
		do
			grep "$MIN_REQ_VER-$MIN_REQ_BUILD-$PREVIOUS_VER-$BUILD_DESCRIPTION" $SYSTEM_UPGRADE_HISTORY_FILE
			HAVE_INSTALLED=$?	
			if [ "$HAVE_INSTALLED" -ne 0 ]
			then
				echo "$MIN_REQ_VER-$MIN_REQ_BUILD-$PREVIOUS_VER-$BUILD_DESCRIPTION" >> $SYSTEM_UPGRADE_HISTORY_FILE
			fi
			PREVIOUS_VER=$[$PREVIOUS_VER+1]
		done
	fi	
}

# We consolidate all restart lighttpd and restart system here.

if [ "$RET_EXIT_INSTALL" -eq 0 ]
then
	# For TMSSS v1.1 Hotfix 1, we should replace VSAPIInfo and libINISetting.so
	# For TMSSS v1.1 Hotfix 2, we should replace VSAPIInfo and libHandlerUtils.so
	echo "Stop lighttdp for 30 seconds." | debug_log
	(sleep 30s ;
	cd / ; 
	/etc/init.d/lighttpd stop ; 
	sleep 5s ; 
	cp -f /var/tmcss/tmp/VSAPIInfo /var/www/iCRC/tmcss/VSAPIInfo ;     
	rm -f /var/tmcss/tmp/VSAPIInfo ;
	chmod 755 /var/www/iCRC/tmcss/VSAPIInfo ; 
	chown webserv:webserv /var/www/iCRC/tmcss/VSAPIInfo ; 
	cp -f /var/tmcss/tmp/libINISetting.so /usr/tmcss/lib/libINISetting.so ;
	rm -f /var/tmcss/tmp/libINISetting.so ;
	chmod 755 /usr/tmcss/lib/libINISetting.so ;
	chown webserv:webserv /usr/tmcss/lib/libINISetting.so ;
	cp -f /var/tmcss/tmp/libHandlerUtils.so /usr/tmcss/lib/libHandlerUtils.so ;
	rm -f /var/tmcss/tmp/libHandlerUtils.so ;
	chmod 755 /usr/tmcss/lib/libHandlerUtils.so ;
	chown webserv:webserv /usr/tmcss/lib/libHandlerUtils.so ;
	ldconfig -v;
	/etc/init.d/lighttpd start)  &
fi

case $RET_EXIT_INSTALL in
# install successful
$ECODE_INS_OK)
    echo `date -R`' - Execute success: '$VAL_EXIT_INSTALL | debug_log  
    update_upgrade_record
    ;;

# reboot required
$ECODE_INS_REBOOT)
    echo `date -R`' - Reboot required: '$VAL_EXIT_INSTALL | debug_log
    update_upgrade_record
    ;;

# install error
$ECODE_INS_ERR)
    echo `date -R`' - Execute failed: '$VAL_EXIT_INSTALL | debug_log
    ;;

# unknown error code
*)
    echo `date -R`' - Unknown exit code: '$VAL_EXIT_INSTALL | debug_log
    ;;
esac

exit $RET_EXIT_INSTALL

