#! /bin/sh -x

# -----------------------------------------------------------------------------
# Define exit codes for ServPatch.sh
# -----------------------------------------------------------------------------

# install.sh is executed successfully.
ECODE_INS_OK=0

# install.sh failed.
ECODE_INS_ERR=1

# reboot is requested by install.sh
ECODE_INS_REBOOT=2

# -----------------------------------------------------------------------------
# function: debug_log
#
# @stdin message
# @stdout message
# -----------------------------------------------------------------------------

debug_log () {
    tee -a install.log
}

# -----------------------------------------------------------------------------
# Generate temp log
# -----------------------------------------------------------------------------

echo `date -R` | debug_log
echo | debug_log

# -----------------------------------------------------------------------------
# In GM build to Patch 1 case, ServPatch.sh could not prevent patch and AU running 
# at the same time. Therefore, we should check if AU is already running in this 
# script and abort patch process if there is AU running.
# -----------------------------------------------------------------------------
ECODE_RUNNING_AU=121
PID_PATCH_FILE=/var/tmcss/pids/au.pid
AU_MODULE_NAME=AU.exe

if [ -e $PID_PATCH_FILE ] 
then
	OLD_PID=`cat $PID_PATCH_FILE`
	FIND_PID_AU=`ps aux | grep $OLD_PID | grep $AU_MODULE_NAME`
	if [ "X$FIND_PID_AU" != "X" ]
	then
		echo "[ERROR] $AU_MODULE_NAME (" $OLD_PID ") is running. Please retry again after $AU_MODULE_NAME is finished." | debug_log
		exit $ECODE_RUNNING_AU
	fi
fi

# -----------------------------------------------------------------------------
# Check if postgresql is running, otherwise we cannot create iCRCLog database.
# -----------------------------------------------------------------------------
echo `date -R` "- Check postgresql is ready by \"service postgresql status\" command." | debug_log
service postgresql status
if [ $? -ne 0 ]
then
	echo "[ERROR] Postgresql is not running, according to \"service postgresql status\" command. Please make sure postgresql is running and try again." | debug_log
	exit $ECODE_INS_ERR
fi

#-----------------------------------------------------------------------------
# Use psql command to make sure postgresql is ready.
#-----------------------------------------------------------------------------

VAL_DB_RUNNING=0
# 1. Change /etc/sudoers for executing "sudo" from non-tty
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/Defaults    requiretty/#Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

RETRY_COUNT=0
while [ $RETRY_COUNT -le 60 ]
do
        echo `date -R` "- Check postgresql status by connecting to postgresql using \"psql\" command. (retry $RETRY_COUNT times)." | debug_log
        sudo -u postgres psql -c "select CURRENT_DATE"
        if [ $? -eq 0 ]
        then
                echo "Postgresql is ready." | debug_log
                VAL_DB_RUNNING=1
                break
        fi
        sleep 2s
        RETRY_COUNT=$(( $RETRY_COUNT + 1))
done

# 3. Restore /etc/sudoers
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/#Defaults    requiretty/Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

if [ "X$VAL_DB_RUNNING" = "X0"  ]
then
        echo "[ERROR] Postgresql is not running, according to \"psql\" command. Please make sure postgresql is running and try again." | debug_log
        exit $ECODE_INS_ERR
fi

# -----------------------------------------------------------------------------
# Copy files to destination folder
# -----------------------------------------------------------------------------
mv /usr/tmcss/bin/LogImport /usr/tmcss/bin/LogImport.old
cp -f LogImport /usr/tmcss/bin/LogImport
chmod 755 /usr/tmcss/bin/LogImport
chown webserv:webserv /usr/tmcss/bin/LogImport

# -----------------------------------------------------------------------------
# Drop and re-create databases
# -----------------------------------------------------------------------------

# 2. Change /etc/sudoers for executing "sudo" from non-tty
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/Defaults    requiretty/#Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

# 8. Restore /etc/sudoers
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/#Defaults    requiretty/Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

# ----------------------------------------------------------------------------
# Put /var/tmcss/tmp/PostInstallMsg.en_US.UTF-8 manually in GM to Patch 1 case
# ----------------------------------------------------------------------------
rm -f /var/tmcss/tmp/PostInstallMsg.*
cp -f PostInstallMsg.* /var/tmcss/tmp/

# -----------------------------------------------------------------------------
# Place other commands here.
# -----------------------------------------------------------------------------

echo "Copy VSAPIInfo and libHandlerUtils.so to tmp folder." | debug_log

cp -f VSAPIInfo /var/tmcss/tmp/VSAPIInfo
cp -f libHandlerUtils.so /var/tmcss/tmp/libHandlerUtils.so

# -----------------------------------------------------------------------------
# All thing are done, and see if there is a need to inform user
# for rebooting Linux.
# -----------------------------------------------------------------------------

echo 'Exit code: '$ECODE_INS_OK | debug_log
exit $ECODE_INS_OK

# echo 'Exit code: '$ECODE_INS_REBOOT | debug_log
# exit $ECODE_INS_REBOOT
