#! /bin/sh -x

# -----------------------------------------------------------------------------
# Define exit codes for ServPatch.sh
# -----------------------------------------------------------------------------

# install.sh is executed successfully.
ECODE_INS_OK=0

# install.sh failed.
ECODE_INS_ERR=1

# reboot is requested by install.sh
ECODE_INS_REBOOT=2

# -----------------------------------------------------------------------------
# function: debug_log
#
# @stdin message
# @stdout message
# -----------------------------------------------------------------------------

debug_log () {
    tee -a install.log
}

# -----------------------------------------------------------------------------
# Generate temp log
# -----------------------------------------------------------------------------

echo `date -R` | debug_log
echo | debug_log

# -----------------------------------------------------------------------------
# In GM build to Patch 1 case, ServPatch.sh could not prevent patch and AU running 
# at the same time. Therefore, we should check if AU is already running in this 
# script and abort patch process if there is AU running.
# -----------------------------------------------------------------------------
ECODE_RUNNING_AU=121
PID_PATCH_FILE=/var/tmcss/pids/au.pid
AU_MODULE_NAME=AU.exe

if [ -e $PID_PATCH_FILE ] 
then
	OLD_PID=`cat $PID_PATCH_FILE`
	FIND_PID_AU=`ps aux | grep $OLD_PID | grep $AU_MODULE_NAME`
	if [ "X$FIND_PID_AU" != "X" ]
	then
		echo "[ERROR] $AU_MODULE_NAME (" $OLD_PID ") is running. Please retry again after $AU_MODULE_NAME is finished." | debug_log
		exit $ECODE_RUNNING_AU
	fi
fi

# -----------------------------------------------------------------------------
# Check if postgresql is running, otherwise we cannot create iCRCLog database.
# -----------------------------------------------------------------------------
echo `date -R` "- Check postgresql is ready by \"service postgresql status\" command." | debug_log
service postgresql status
if [ $? -ne 0 ]
then
	echo "[ERROR] Postgresql is not running, according to \"service postgresql status\" command. Please make sure postgresql is running and try again." | debug_log
	exit $ECODE_INS_ERR
fi

#-----------------------------------------------------------------------------
# Use psql command to make sure postgresql is ready.
#-----------------------------------------------------------------------------

VAL_DB_RUNNING=0
# 1. Change /etc/sudoers for executing "sudo" from non-tty
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/Defaults    requiretty/#Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

RETRY_COUNT=0
while [ $RETRY_COUNT -le 60 ]
do
        echo `date -R` "- Check postgresql status by connecting to postgresql using \"psql\" command. (retry $RETRY_COUNT times)." | debug_log
        sudo -u postgres psql -c "select CURRENT_DATE"
        if [ $? -eq 0 ]
        then
                echo "Postgresql is ready." | debug_log
                VAL_DB_RUNNING=1
                break
        fi
        sleep 2s
        RETRY_COUNT=$(( $RETRY_COUNT + 1))
done

# 3. Restore /etc/sudoers
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/#Defaults    requiretty/Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

if [ "X$VAL_DB_RUNNING" = "X0"  ]
then
        echo "[ERROR] Postgresql is not running, according to \"psql\" command. Please make sure postgresql is running and try again." | debug_log
        exit $ECODE_INS_ERR
fi

# -----------------------------------------------------------------------------
# change owner of LogImport.ini
# -----------------------------------------------------------------------------
echo "Change owner of LogImport.ini" | debug_log
chown webserv:webserv /var/tmcss/conf/LogImport.ini

# -----------------------------------------------------------------------------
# Copy files to destination folder
# -----------------------------------------------------------------------------
mv /usr/tmcss/bin/PtnImport /usr/tmcss/bin/PtnImport.old
cp -f PtnImport /usr/tmcss/bin/PtnImport
chmod 755 /usr/tmcss/bin/PtnImport
chown webserv:webserv /usr/tmcss/bin/PtnImport

cp -f drop_table_icrc.sh /usr/tmcss/bin/drop_table_icrc.sh
chmod 755 /usr/tmcss/bin/drop_table_icrc.sh
chown webserv:webserv /usr/tmcss/bin/drop_table_icrc.sh

cp -f drop_table_icrc2.sh /usr/tmcss/bin/drop_table_icrc2.sh
chmod 755 /usr/tmcss/bin/drop_table_icrc2.sh
chown webserv:webserv /usr/tmcss/bin/drop_table_icrc2.sh

cp -f drop_table_icrclog.sh /usr/tmcss/bin/drop_table_icrclog.sh
chmod 755 /usr/tmcss/bin/drop_table_icrclog.sh
chown webserv:webserv /usr/tmcss/bin/drop_table_icrclog.sh

cp -f AUInit.exe /usr/tmcss/bin/AUInit.exe
chmod 755 /usr/tmcss/bin/AUInit.exe
chown webserv:webserv /usr/tmcss/bin/AUInit.exe

cp -f AU.exe /usr/tmcss/bin/AU.exe
chmod 755 /usr/tmcss/bin/AU.exe
chown webserv:webserv /usr/tmcss/bin/AU.exe

cp -f MakeBF /usr/tmcss/bin/MakeBF
chmod 755 /usr/tmcss/bin/MakeBF
chown webserv:webserv /usr/tmcss/bin/MakeBF

cp -f GenCRCDiff /usr/tmcss/bin/GenCRCDiff
chmod 755 /usr/tmcss/bin/GenCRCDiff
chown webserv:webserv /usr/tmcss/bin/GenCRCDiff

cp -f UpdatePattern.sh /usr/tmcss/bin/UpdatePattern.sh
chmod 755 /usr/tmcss/bin/UpdatePattern.sh
chown webserv:webserv /usr/tmcss/bin/UpdatePattern.sh

cp -f CombineIni.py /usr/tmcss/bin/CombineIni.py
chmod 755 /usr/tmcss/bin/CombineIni.py
chown webserv:webserv /usr/tmcss/bin/CombineIni.py

cp -f updates.php /var/www/AdminUI/php/updates.php
chmod 755 /var/www/AdminUI/php/updates.php
chown webserv:webserv /var/www/AdminUI/php/updates.php

cp -f AUUtility.php /var/www/AdminUI/php/inc/AUUtility.php
chmod 755 /var/www/AdminUI/php/inc/AUUtility.php
chown webserv:webserv /var/www/AdminUI/php/inc/AUUtility.php

mv /var/tmcss/conf/UpdatePattern.ini /var/tmcss/conf/UpdatePattern.ini.old
cp -f UpdatePattern.ini /var/tmcss/conf/UpdatePattern.ini
chmod 644 /var/tmcss/conf/UpdatePattern.ini
chown webserv:webserv /var/tmcss/conf/UpdatePattern.ini

python /usr/tmcss/bin/CombineIni.py /var/tmcss/conf/UpdatePattern.ini.old /var/tmcss/conf/UpdatePattern.ini

# -----------------------------------------------------------------------------
# Drop and re-create databases
# -----------------------------------------------------------------------------

# 1. Remove log files under /var/lib/pgsql/data/pg_log/ 
#
rm -f /var/lib/pgsql/data/pg_log/*

# 2. Change /etc/sudoers for executing "sudo" from non-tty
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/Defaults    requiretty/#Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

# 3. Get current active database
#
ACTIVE_IDX=$(/usr/tmcss/bin/DumpMmap | grep ActiveIdx | cut -d " " -f 2)

# 4. Drop and re-create inactive database
#
if [ "X$ACTIVE_IDX" = "X0"  ]
then
        echo "ActiveIdx: 0, Run drop_table_icrc2.sh to drop and re-create inactive iCRC2 database." | debug_log
	/usr/tmcss/bin/drop_table_icrc2.sh
else
	echo "ActiveIdx: 1, Run drop_table_icrc.sh to drop and re-create inactive iCRC database." | debug_log
	/usr/tmcss/bin/drop_table_icrc.sh
fi

# 5. Drop and re-create iCRCLog database
echo "Run drop_table_icrclog.sh to drop and re-create iCRCLog." | debug_log
/usr/tmcss/bin/drop_table_icrclog.sh

LATEST_PATTERN_FULL_VERSION=`cat /var/tmcss/conf/PatternVersion.ini | grep 'LeaderTablePatternVersion' | cut -d "=" -f 2`
LATEST_PATTERN_VERSION=`echo $LATEST_PATTERN_FULL_VERSION | awk '{print substr($0, length($0) - 4, 3); }'`
FILE_LATEST_PATTERN=/var/tmcss/activeupdate/icrc\$tbl.$LATEST_PATTERN_VERSION
echo "Pattern full version: $LATEST_PATTERN_FULL_VERSION" | debug_log
echo "Pattern file path   : $FILE_LATEST_PATTERN" | debug_log

if [ ! -r $FILE_LATEST_PATTERN ]
then
        echo "[ERROR] Can not find pattern" $FILE_LATEST_PATTERN | debug_log
        exit $ECODE_INS_ERR
fi

echo "PtnImport with (" $FILE_LATEST_PATTERN ") into database" | debug_log
sudo -u webserv /usr/tmcss/bin/PtnImport localhost $FILE_LATEST_PATTERN &> /dev/null
if [ $? -ne 0 ]
then
        echo "[ERROR] PtnImport failed. Failed to import latest pattern into database." | debug_log
        exit $ECODE_INS_ERR
fi

echo "LogImport vinfo of the latest pattern (" $FILE_LATEST_PATTERN ") into database" | debug_log
sudo -u webserv /usr/tmcss/bin/LogImport -conf /var/tmcss/conf/LogImport.ini -au $FILE_LATEST_PATTERN &> /dev/null
if [ $? -ne 0 ]
then
        echo "[ERROR] LogImport failed. Failed to import latest pattern's vinfo into database." | debug_log
        exit $ECODE_INS_ERR
fi

echo "UpdateMmap to (" $LATEST_PATTERN_FULL_VERSION ")" | debug_log
sudo -u webserv /usr/tmcss/bin/UpdateMmap $LATEST_PATTERN_FULL_VERSION &> /dev/null
if [ $? -ne 0 ]
then
        echo "[ERROR] UpdateMmap failed. Failed to update Mmap." | debug_log
        exit $ECODE_INS_ERR
fi

# 6. Get current active database
ACTIVE_IDX=$(/usr/tmcss/bin/DumpMmap | grep ActiveIdx | cut -d " " -f 2)

# 7. Drop and re-create inactive database
if [ "X$ACTIVE_IDX" = "X0"  ]
then
        echo "ActiveIdx: 0, Run drop_table_icrc2.sh to drop and re-create inactive iCRC2 database." | debug_log
	/usr/tmcss/bin/drop_table_icrc2.sh
else
	echo "ActiveIdx: 1, Run drop_table_icrc.sh to drop and re-create inactive iCRC database." | debug_log
	/usr/tmcss/bin/drop_table_icrc.sh
fi

# 8. Restore /etc/sudoers
#
mv /etc/sudoers /etc/sudoers.tmp
cat /etc/sudoers.tmp | sed 's/#Defaults    requiretty/Defaults    requiretty/' > /etc/sudoers
chmod 0440 /etc/sudoers
rm -f /etc/sudoers.tmp

# ----------------------------------------------------------------------------
# Put /var/tmcss/tmp/PostInstallMsg.en_US.UTF-8 manually in GM to Patch 1 case
# ----------------------------------------------------------------------------
rm -f /var/tmcss/tmp/PostInstallMsg.*
cp -f PostInstallMsg.* /var/tmcss/tmp/

# -----------------------------------------------------------------------------
# Place other commands here.
# -----------------------------------------------------------------------------

echo "Copy VSAPIInfo and libINISetting.so to tmp folder." | debug_log

cp -f VSAPIInfo /var/tmcss/tmp/VSAPIInfo
cp -f libINISetting.so /var/tmcss/tmp/libINISetting.so


# -----------------------------------------------------------------------------
# All thing are done, and see if there is a need to inform user
# for rebooting Linux.
# -----------------------------------------------------------------------------

echo 'Exit code: '$ECODE_INS_OK | debug_log
exit $ECODE_INS_OK

# echo 'Exit code: '$ECODE_INS_REBOOT | debug_log
# exit $ECODE_INS_REBOOT
